@@ -1,5 +1,5 @@
 if(getRversion() >= "2.15.1") {
-  utils::globalVariables(c("X1", "X2", "Y1", "Y2", "group", "id"))
+  utils::globalVariables(c("X1", "X2", "Y1", "Y2", "group", "id", "midX", "midY"))
 }
 
 
@@ -9,7 +9,8 @@ if(getRversion() >= "2.15.1") {
 #'
 #' @export
 #' @param net an object of class \code{igraph} or \code{network}. If the object is of class \code{igraph}, the \link[intergraph:asNetwork]{intergraph} package is used to convert it to class \code{network}.
-#' @param mode a placement method from the list of modes provided in the \link[sna:gplot.layout]{sna} package. Defaults to the Fruchterman-Reingold force-directed algorithm.
+#' @param mode a placement method from the list of modes provided in the \link[sna:gplot.layout]{sna} package. Defaults to the Fruchterman-Reingold force-directed algorithm. If \code{mode} is set to \code{"geo"} and \code{net} contains two vertex attributes called \code{"lat"} and \code{"lon"}, these are used instead for geographic networks.
+#' @param layout.par options to the placement method, as listed in \link[sna]{gplot.layout}.
 #' @param size size of the network nodes. Defaults to 12. If the nodes are weighted, their area is proportionally scaled up to the size set by \code{size}.
 #' @param alpha a level of transparency for nodes, vertices and arrows. Defaults to 0.75.
 #' @param weight.method a weighting method for the nodes. Accepts \code{"indegree"}, \code{"outdegree"} or \code{"degree"} (the default). Set to \code{"none"} to plot unweighted nodes.
@@ -19,9 +20,11 @@ if(getRversion() >= "2.15.1") {
 #' @param node.alpha transparency of the nodes. Inherits from \code{alpha}.
 #' @param segment.alpha transparency of the vertex links. Inherits from \code{alpha}.
 #' @param segment.color color of the vertex links. Defaults to \code{"grey"}.
-#' @param segment.size size of the vertex links. Defaults to 0.25.
-#' @param arrow.size size of the vertex arrows for directed network plotting. Defaults to 0.
-#' @param label.nodes label nodes with their vertex attributes. If set to \code{TRUE}, all nodes are labelled. Also accepts a vector of character strings to match with vertex names.
+#' @param segment.size size of the vertex links, as a vector of values or as a single value. Defaults to 0.25.
+#' @param segment.label labels for the vertex links at mid-edges. Label size will be set to 1 / \code{segment.size}, and label alpha will inherit from \code{alpha}.
+#' @param arrow.size size of the vertex arrows for directed network plotting, in centimeters. Defaults to 0.
+#' @param label.nodes label nodes with their vertex names attribute. If set to \code{TRUE}, all nodes are labelled. Also accepts a vector of character strings to match with vertex names.
+#' @param label.size size of the labels.  Defaults to \code{size / 2}.
 #' @param top8.nodes use the top 8 nodes as node groups, colored with \code{"Set1"}. The rest of the network will be plotted as the ninth (grey) group. Experimental.
 #' @param trim.labels removes '@@', 'http://', 'www.' and the ending '/' from vertex names. Cleans up labels for website and Twitter networks. Defaults to \code{TRUE}.
 #' @param quantize.weights break node weights to quartiles. Fails when quartiles do not uniquely identify nodes.
@@ -30,6 +33,7 @@ if(getRversion() >= "2.15.1") {
 #' @param ... other arguments supplied to geom_text for the node labels. Arguments pertaining to the title or other items can be achieved through ggplot2 methods.
 #' @seealso \code{\link[sna]{gplot}} in the \link[sna:gplot]{sna} package
 #' @author Moritz Marbach \email{mmarbach@@mail.uni-mannheim.de} and Francois Briatte \email{f.briatte@@ed.ac.uk}
+#' @details The \code{weight.method} argument produces visually scaled nodes that are proportionally sized to their unweighted degree. To compute weighted centrality or degree measures, see Tore Opsahl's \code{\link[tnet]{tnet}} package.
 #' @importFrom grid arrow
 #' @examples
 #' require(network)
@@ -41,32 +45,33 @@ if(getRversion() >= "2.15.1") {
 #' dimnames(nw.mat)   <- list(1:x, 1:x)
 #' nw.mat[row(nw.mat) != col(nw.mat)] <- runif(ndyads) < density
 #' nw.mat
-#' rnd <- network(nw.mat)
+#' rnd <- network::network(nw.mat)
 #' rnd
 #'
 #' # random network
-#' pRnd <- ggnet(rnd, label = TRUE, alpha = 1, color = "white", segment.color = "grey10")
+#' pRnd <- ggnet(rnd, label.nodes = TRUE, alpha = 1, color = "white", segment.color = "grey10")
 #' # pRnd
 #'
 #' # random groups
 #' category = LETTERS[rbinom(x, 4, .5)]
-#' ggnet(rnd, label = TRUE, color = "white", segment.color = "grey10", node.group = category)
+#' ggnet(rnd, label.nodes = TRUE, color = "white", segment.color = "grey10", node.group = category)
 #'
-#' # City and service firms data from the UCIrvine Network Data Repository.
+#' # city and service firms data from the UCIrvine Network Data Repository
 #' url = url("http://networkdata.ics.uci.edu/netdata/data/cities.RData")
 #' print(load(url))
 #' close(url)
+#'
 #' # plot cities, firms and law firms
-#' type = network::get.vertex.attribute(cities, "type")
+#' type = cities %v% "type"
 #' type = ifelse(grepl("City|Law", type), gsub("I+", "", type), "Firm")
 #' pRnd <- ggnet(cities, mode = "kamadakawai", alpha = .2, node.group = type,
-#'       label = c("Paris", "Beijing", "Chicago"), color = "darkred")
+#'       label.nodes = c("Paris", "Beijing", "Chicago"), color = "darkred")
 #' # pRnd
 
-
 ggnet <- function(
   net,                          # an object of class network
   mode             = "fruchtermanreingold", # placement algorithm
+  layout.par       = NULL,      # placement options
   size             = 12,        # node size
   alpha            = .75,       # transparency
   weight.method    = "none",    # what to weight the nodes with: "freeman", "indegree", "outdegree"
@@ -76,9 +81,11 @@ ggnet <- function(
   node.alpha       = NULL,      # transparency for nodes (inherits from alpha)
   segment.alpha    = NULL,      # transparency for links (inherits from alpha)
   segment.color    = "grey",    # default links are rgb(190, 190, 190)
+  segment.label    = NULL,      # label network at mid-edges
   segment.size     = .25,       # set to 0 to remove from plot
   arrow.size       = 0,         # set to 0 to remove from plot
   label.nodes      = FALSE,     # add vertex names in small print; can be a list of vertex names
+  label.size       = size / 2,         # size of the vertex names
   top8.nodes       = FALSE,     # color the top 8 nodes by weighting method with ColorBrewer Set1
   trim.labels      = TRUE,      # clean vertex names
   quantize.weights = FALSE,     # break weights to quartiles
@@ -86,19 +93,10 @@ ggnet <- function(
   legend.position  = "right",   # set to "none" to remove from plot
   ...                           # passed to geom_text for node labels
 ){
-  require(intergraph)    # igraph conversion
-  require(network)       # vertex attributes
-  require(RColorBrewer)  # default colors
-  require(sna)           # placement algorithm
-
-  # get arguments
-  weight    = c("indegree", "outdegree")
-  weight    = ifelse(weight.method %in% weight, weight.method, "freeman")
-  quartiles = quantize.weights
-  labels    = label.nodes
-
-  # alpha default
-  inherit <- function(x) ifelse(is.null(x), alpha, x)
+  require(intergraph   , quietly = TRUE) # igraph conversion
+  require(network      , quietly = TRUE) # vertex attributes
+  require(RColorBrewer , quietly = TRUE) # default colors
+  require(sna          , quietly = TRUE) # placement algorithm
 
   # support for igraph objects
   if(class(net) == "igraph") {
@@ -107,6 +105,18 @@ ggnet <- function(
   if(class(net) != "network")
     stop("net must be a network object of class 'network' or 'igraph'")
 
+  # vertex attributes for weight detection
+  vattr = network::list.vertex.attributes(net)
+
+  # get arguments
+  weight    = c("indegree", "outdegree", vattr)
+  weight    = ifelse(weight.method %in% weight | length(weight.method) > 1,
+                     weight.method, "freeman")
+  quartiles = quantize.weights
+  labels    = label.nodes
+
+  # alpha default
+  inherit <- function(x) ifelse(is.null(x), alpha, x)
   # subset
   if(subset.threshold > 0) {
     network::delete.vertices(
@@ -117,13 +127,29 @@ ggnet <- function(
   # get sociomatrix
   m <- as.matrix.network.adjacency(net)
 
-  # get coordinates placement algorithm
-  placement <- paste0("gplot.layout.", mode)
-  if(!exists(placement)) stop("Unsupported placement method.")
+  if(mode == "geo" & all(c("lat", "lon") %in% vattr)) {
 
-  plotcord <- do.call(placement, list(m, NULL))
-  plotcord <- data.frame(plotcord)
-  colnames(plotcord) = c("X1", "X2")
+    plotcord = data.frame(
+      X1 = as.numeric(net %v% "lon"),
+      X2 = as.numeric(net %v% "lat")
+      )
+
+    # remove outliers
+    plotcord$X1[ abs(plotcord$X1) > quantile(abs(plotcord$X1), .9, na.rm = TRUE) ] = NA
+    plotcord$X2[ is.na(plotcord$X1) | abs(plotcord$X2) > quantile(abs(plotcord$X2), .9, na.rm = TRUE) ] = NA
+    plotcord$X1[ is.na(plotcord$X2) ] = NA
+
+  } else {
+
+    # get coordinates placement algorithm
+    placement <- paste0("gplot.layout.", mode)
+    if(!exists(placement)) stop("Unsupported placement method.")
+
+    plotcord <- do.call(placement, list(m, layout.par))
+    plotcord <- data.frame(plotcord)
+    colnames(plotcord) = c("X1", "X2")
+
+  }
 
   # get edgelist
   edglist <- as.matrix.network.edgelist(net)
@@ -143,6 +169,19 @@ ggnet <- function(
   )
   degrees$freeman <- with(degrees, indegree + outdegree)
 
+  # custom weights: vector of weights
+  if(length(weight.method) == network.size(net)) {
+    degrees$user = weight.method
+    weight = "user"
+  }
+
+  # custom weights: vertex attribute
+  if(weight.method %in% vattr) {
+    degrees$user = net %v% weight.method
+    names(degrees)[ ncol(degrees) ] = weight.method
+    weight = weight.method
+  }
+
   # trim vertex names
   if(trim.labels) {
     degrees$id = gsub("@|http://|www.|/$", "", degrees$id)
@@ -168,7 +207,7 @@ ggnet <- function(
       plotcord$id = ""
     }
   } else {
-    plotcord$id[-which(plotcord$id %in% labels)] = ""
+    plotcord$id[ -which(plotcord$id %in% labels) ] = ""
   }
 
   # get vertice midpoints (not -yet- used later on)
@@ -190,8 +229,20 @@ ggnet <- function(
       )
     )
 
+  # label mid-edges
+  if(!is.null(segment.label) & length(segment.label) == nrow(edges)) {
+    pnet <- pnet + geom_text(
+      aes(x = midX, y = midY),
+      data   = edges,
+      label = segment.label,
+      size = 1 / segment.size, # fixed setting
+      colour = segment.color,
+      alpha  = inherit(segment.alpha)
+    )
+  }
+
   # null weighting
-  if(weight.method == "none") {
+  if(weight.method == c("none")) {
     pnet <- pnet + geom_point(
       data  = plotcord,
       alpha = inherit(node.alpha),
@@ -199,11 +250,12 @@ ggnet <- function(
     )
   }
   else {
-    plotcord$weight <- degrees[, which(names(degrees) == weight)]
+
+    plotcord$weight <- degrees[, weight ]
 
     # show top weights
     cat(nrow(plotcord), "nodes, weighted by", weight, "\n\n")
-    print(head(degrees[order(-degrees[weight]), ]))
+    print(head(degrees[ order( -degrees[weight] ), ]))
 
     # proportional scaling
     sizer <- scale_size_area(names[2], max_size = size)
@@ -233,7 +285,7 @@ ggnet <- function(
   }
 
   # default colors
-  n = length(unique(na.omit(node.group)))
+  n = length(unique(suppressWarnings(na.omit(node.group))))
   if(length(node.color) != n & !is.null(node.group)) {
     warning("Node groups and node colors are of unequal length; using default colors.")
     if(n > 0 & n < 10) {
@@ -248,13 +300,13 @@ ggnet <- function(
       scale_colour_manual(
         names[1],
         values = node.color,
-        guide  = guide_legend(override.aes = list(size = 1 + sqrt(size)))
+        guide  = guide_legend(override.aes = list(size = label.size))
       )
   }
 
   # add text labels
   if(length(unique(plotcord$id)) > 1 | unique(plotcord$id)[1] != "") {
-    pnet <- pnet + geom_text(aes(label = id), size = 1 + sqrt(size), ...)
+    pnet <- pnet + geom_text(aes(label = id), size = label.size, ...)
   }
 
   # finalize: remove grid, axes and scales
